' ================================================================================================
'  Theremino System - ReleDriver V4
' ------------------------------------------------------------------------------------------------
'  - ENG - 
'  This script sends data to a shift register that controls a bank of relays (or other devices)
'  ENABLE, DATA, CLOCK And 5Volt signals are sent via a Master Module and four shielded wires
'  to receiver modules at great diostance (hundred of meters)
'  A suitable hardware here: http://www.theremino.com/hardware/outputs/actuators#releswitcherv2
' ------------------------------------------------------------------------------------------------
'  - ITA - 
'  Questo script manda i dati a uno shift register che pilota un banco di relè (o altro) 
'  I segnali ENABLE, DATA, CLOCK e 5VOLT sono inviati via Master e cavo schermato a quattro fili
'  a moduli ricevitori situati a grande distanza (centinaia di metri).
'  Un esempio di hardware qui: http://www.theremino.com/hardware/outputs/actuators#releswitcherv2
' ================================================================================================

Const NumRele As Int32 = 32					' Rele number			- Numero di RELE (8,16,24...)
Const LabelSize As Int32 = 16				' Label SIZE			- Larghezza delle caselle

Const SlotEnable As Int32 = 1				' Slot ENABLE			- Slot del filo ENABLE
Const SlotData As Int32 = 2					' Slot DATA				- Slot del filo DATA
Const SlotClock As Int32 = 3				' Slot CLOCK			- Slot del filo CLOCK

Const DelayMillisec As Int32 = 4			' Time For Each bit		- Millisecondi per ogni bit

Dim ColorUsed As Color = Color.Orange		' Color For RELE USED	- Colore per RELE USATO
Dim ColorON As Color = Color.Red			' Color For RELE On		- Colore per RELE ACCESO
Dim ColorArmed As Color = Color.Yellow		' Color For ARMED		- Colore per CASELLA ARMATA
Dim ColorUnused As Color = Color.White		' Color For UNUSED		- Colore per CASELLA NON USATA
Dim ColorBack As Color = Color.LightYellow	' Color For BACK		- Colore per SFONDO FINESTRA 
Dim ColorText As Color = Color.Black		' Color For TEXT		- Colore per TESTO



' ================================================================================================
'  - ENG - THE SIMPLIFIED MANAGEMENT ENDS HERE - MODIFY THE FOLLOWING CAREFULLY
' ------------------------------------------------------------------------------------------------
'  - ITA - LA GESTIONE SEMPLIFICATA FINISCE QUI - MODIFICARE LA PARTE SEGUENTE CON ATTENZIONE
' ================================================================================================

' ------------------------------------------------------------------------------------------------
'  Main Loop
' ------------------------------------------------------------------------------------------------
Sub Main(ByVal s() As String)
	If Form1 IsNot Nothing Then Return
	Form1 = New form
	InitializeComponents
	MemoryMappedFile_Init
	timeBeginPeriod(1)
	SendEmptySequence
	While Form1.Visible
		Application.DoEvents
		Threading.Thread.Sleep(10)
	End While
End Sub

Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Form1.Closing
	SendEmptySequence
	timeEndPeriod(1)
End Sub

Dim ValidKeyDown As Boolean = True
Sub Form1_KeyDown(sender As Object, e As System.Windows.Forms.KeyEventArgs) Handles Form1.KeyDown
	If e.KeyCode = Keys.Space And ValidKeyDown Then
		ValidKeyDown = False
		Pressed_Start
	End If
End Sub
Sub Form1_KeyUp(sender As Object, e As System.Windows.Forms.KeyEventArgs) Handles Form1.KeyUp
	If e.KeyCode = Keys.Space Then
		ValidKeyDown = True
	End If
End Sub

Sub Button1_GotFocus(sender As Object, e As System.EventArgs) Handles Button1.GotFocus
	Label1.Focus
End Sub

Sub Button1_MouseDown(sender As Object, e As MouseEventArgs) Handles Button1.MouseDown
	Pressed_Start
End Sub

Sub Button2_MouseDown(sender As Object, e As MouseEventArgs) Handles Button2.MouseDown
	Pressed_Reset
End Sub


' ---------------------------------------------------------------------- User Interface Components
Dim WithEvents Form1 As Form
Dim Label1 As Label = New Label
Dim LabelArray(NumRele - 1) As Label
Dim WithEvents Button1 As Button = New Button
Dim WithEvents Button2 As Button = New Button

Sub InitializeComponents()
	Application.EnableVisualStyles
	form1.SuspendLayout
	' ------------------------------------------------------------------ Labels
	InitializeLabels
	' ------------------------------------------------------------------ Form
	Form1.Text = "Theremino Script - Rele Driver V4"
	Form1.ClientSize = New System.Drawing.Size(LabelArray(NumRele - 1).right + 12, 230)
	Form1.StartPosition = FormStartPosition.CenterScreen
	Form1.FormBorderStyle = FormBorderStyle.Fixed3D
	Form1.MaximizeBox = False
	Form1.BackColor = ColorBack
	Form1.KeyPreview = True
	' ------------------------------------------------------------------ Label 1
	Label1.BorderStyle = BorderStyle.Fixed3D
	Label1.ForeColor = ColorText
	Label1.BackColor = ColorUnused 
	Label1.Location = New Point(10, 10)
	Label1.Font = New Font("Arial", 12 )
	Label1.Size = New Size(Form1.ClientSize.Width - 20, 21)
	Form1.Controls.Add(Label1)
	' ------------------------------------------------------------------ Button 1
	Button1.BackColor = ColorUnused
	Button1.ForeColor = ColorText
	Button1.Location = New Point(10, 172)
	Button1.Font = New Font("Arial", 12)
	Button1.Size = New Size(Form1.ClientSize.Width - 100, 50)
	Button1.Text = "START" & vbcrlf & "(or press key SPACE on keyboard)"
	Button1.TabStop = False
	Form1.Controls.Add(Button1)
	' ------------------------------------------------------------------ Button 2
	Button2.BackColor = ColorUnused
	Button2.ForeColor = ColorText
	Button2.Location = New Point(Form1.ClientSize.Width - 86, 172)
	Button2.Font = New Font("Arial", 12)
	Button2.Size = New Size(76, 50)
	Button2.Text = "Reset"
	Button2.TabStop = False
	Form1.Controls.Add(Button2)
	' ------------------------------------------------------------------ 
	Form1.ResumeLayout
	Form1.PerformLayout()
	Form1.Show
End Sub

Public Delegate Sub ClickHandlerDelegate (sender As Object, e As EventArgs)
Dim instance As New ClickHandlerDelegate(AddressOf ClickHandler)
Sub ClickHandler(sender As Object, e As EventArgs)
	Dim l As Label = CType(sender, Label)
	Dim n As Int32 = CInt(Val(l.Text))
	If l.BackColor <> ColorON And l.BackColor <> ColorUsed Then
		If l.BackColor <> ColorArmed Then
			l.BackColor = ColorArmed 
			Label1.Text = "Rele " & n.ToString & " ready"
			'My.Computer.Audio.PlaySystemSound(Media.SystemSounds.Beep)
		Else
			l.BackColor = ColorUnused 
			Label1.Text = "Rele " & n.ToString & " disabled"
			'My.Computer.Audio.PlaySystemSound(Media.SystemSounds.Exclamation)
		End If
	End If
End Sub

Sub InitializeLabels()
	For i As Int32 = 0 To LabelArray.Length - 1
		LabelArray(i) = New Label
		Form1.Controls.Add(LabelArray(i))
		With LabelArray(i)
			.ForeColor = ColorText
			.BackColor = ColorUnused 
			.BorderStyle = BorderStyle.Fixed3D
			.Size = New Size(LabelSize, 120)
			.Location = New Point(10 + (LabelSize + 2) * i, 40)
			.Font = New Font("Arial", 12)
			.Text = (i + 1).tostring
			.TextAlign = System.Drawing.ContentAlignment.TopCenter
			Addhandler .Click, AddressOf ClickHandler
		End With
	Next
End Sub

Sub Pressed_Reset
	'My.Computer.Audio.PlaySystemSound(Media.SystemSounds.Asterisk)
	SendEmptySequence
End Sub

Sub SendEmptySequence
	Dim Sequence As String = ""
	For i As Int32 = 0 To NumRele -1
		With LabelArray(i)
			.BackColor = ColorUnused
			Sequence = Sequence & "0"
		End With
	Next
	SendShiftSequence(Sequence)
End Sub

Sub Pressed_Start
	My.Computer.Audio.PlaySystemSound(Media.SystemSounds.Asterisk)
	Dim Sequence As String = ""
	For i As Int32 = 0 To NumRele -1
		With LabelArray(i)
			If .BackColor = ColorON Then
				.BackColor = ColorUsed
			End If
			If .BackColor = ColorArmed Then
				Sequence = Sequence & "1"
				.BackColor = ColorON
			Else
				Sequence = Sequence & "0"
			End If 
		End With
	Next
	SendShiftSequence(Sequence)
End Sub

Sub SendShiftSequence (ByVal Sequence As String)
	Label1.text = "Shift sequence = " & Sequence
	Form1.Refresh
	WriteSlot(SlotEnable, 0)
	For i As Int32 = Sequence.length - 1 To 0 Step -1
		If Sequence(i) = "0" Then
			WriteSlot(SlotData, 0)
		Else
			WriteSlot(SlotData, 1000)
		End If
		WriteSlot(SlotClock, 0)
		Threading.Thread.Sleep(DelayMillisec)
		WriteSlot(SlotClock, 1000)
		Threading.Thread.Sleep(DelayMillisec)
	Next
	WriteSlot(SlotEnable, 1000)
	WriteSlot(SlotData, 0)
	WriteSlot(SlotClock, 0)
End Sub

' ===============================================================
'  WINDOWS SCHEDULER PRECISION
' =============================================================== 
Declare Function timeBeginPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
Declare Function timeEndPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32

