' ================================================================================================
' - ENGLISH - VOLT METER
' ================================================================================================
'  This script reads a PIN value and displays the voltage on a large display. 
'  The PIN must be configured as Adc_16 and can directly read voltages up to 3.3 Volts. 
'  With resistive dividers you can measure higher voltages, but the must be ground referenced. 
'  Even without divider add at least one resistor from 10K to 100K in series to the signal.
' ------------------------------------------------------------------------------------------------

' ================================================================================================
' - ITALIANO - MISURATORE DI TENSIONE
' ================================================================================================
'  Questo script legge il valore di uno Pin e visualizza la tensione su un grande display.
'  Il Pin deve essere configurato come Adc_16 e può leggere direttamente tensioni fino a 3.3 Volt.
'  Con partitori resistivi si possono misurare tensioni più alte, ma sempre riferite a massa. 
'  Anche senza partitore aggiungere almeno un resistore da 10K a 100K in serie al segnale.
' ------------------------------------------------------------------------------------------------

' Schematics here: http://www.theremino.com/hardware/inputs/sensors#meter 

' Vmin   Vmax      R1      R2       Coefficient    End of scale
' -------------------------------------------------------------
' 0 V    +3.3 V    10k     --       0.00330          3.30 Volt
' 0 V    +10 V     100k    47k      0.01032         10.32 Volt
' 0 V    +20 V     270k    47k      0.02226         22.26 Volt
' 0 V    +50 V     680k    47k      0.05104         51.04 Volt
' 0 V    +100 V    1 Mega  33k      0.10330        103.30 Volt

Const Coefficient As Single = 0.0033
Const VoltMeterSlot As Int32 = 1

Sub Main(ByVal s() As String)
	If Form1 IsNot Nothing Then Return
	Form1 = New Form
	InitializeComponents
	MemoryMappedFile_Init
	While Form1.Visible
		Application.DoEvents
		Threading.Thread.Sleep(10)
	End While
End Sub

Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Form1.Closing
	Timer1.Stop
End Sub

Sub Timer1_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles Timer1.Tick
	Static Volt As Single =0
	Dim value As Single
	value = ReadSlot(VoltMeterSlot) * Coefficient
	SmoothValue_Pow_Adaptive(volt, value, 20)
	Label1.Text = volt.ToString("0.000")
End Sub


' ================================================================================================
'   An adaptive, HiQuality, smoothing function
' ================================================================================================
Private Sub SmoothValue_Pow_Adaptive(ByRef value As Single, ByVal new_value As Single, ByVal speed As Single)
	Dim delta As Single = new_value - value
	Dim delta_sgn As Single = Math.Sign(delta)
	Dim delta_abs As Single = Math.Abs(delta)
	Dim delta_pow As Single = CSng(delta_abs ^ 2) * speed
	If value <> 0 Then delta_pow /= abs(value)
	If delta_pow >= delta_abs Then
		value = new_value
	Else
		value += delta_sgn * delta_pow
	End If
End Sub

' ================================================================================================
'   User Interface Components
' ================================================================================================
WithEvents Form1 As Form
WithEvents Timer1 As Timer = New Timer
Dim Label1 As Label = New Label

Sub InitializeComponents()
	Application.EnableVisualStyles
	form1.SuspendLayout
	' ------------------------------------------------------------------ Form
	Form1.Text = "Theremino Script - Volt Meter"
	Form1.FormBorderStyle = FormBorderStyle.Fixed3D
	Form1.MaximizeBox = False
	Form1.ClientSize = New System.Drawing.Size(320, 90)
	Form1.MinimumSize = Form1.Size
	Form1.StartPosition = FormStartPosition.CenterScreen
	' ------------------------------------------------------------------ Label 1
	'Label1.BorderStyle = BorderStyle.Fixed3D
	Label1.Location = New Point(0, 0)
	Label1.Size = Form1.ClientSize
	Label1.BackColor = Color.DarkBlue
	Label1.ForeColor = Color.Gold
	Label1.Font = New Font("Tahoma", 55, FontStyle.Bold)
	Label1.TextAlign = ContentAlignment.MiddleRight
	Label1.Anchor = AnchorStyles.Top Or AnchorStyles.Left Or _
					AnchorStyles.Right Or AnchorStyles.Bottom
	Form1.Controls.Add(Label1)
	' ------------------------------------------------------------------ Timer 1
	Timer1.Interval = 15
	Timer1.Start
	' ------------------------------------------------------------------ 
	Form1.ResumeLayout
	Form1.PerformLayout()
	Form1.Show
End Sub
